if(length(find.package("rJava",quiet=TRUE))==0){
  install.packages("rJava")
}

library("rJava")

#######################################
# Common Java classes
labstoryClass <- list(
  Document       = "org/ecobima/labnote/client/document/LabstoryDocument",
  DocumentInfo   = "org/ecobima/labnote/client/document/DocumentInfo",
  DocumentObject = "org/ecobima/labnote/client/document/DocumentObject",
  RLabstory      = "org/ecobima/labnote/client/r/RLabstory",
  RLabstoryUtil  = "org/ecobima/labnote/client/r/RLabstoryUtil"
)


#######################################
# Initialize Labstory
#
# TODO might want to have another config and data directory? or? otherwise it will
# stall the local installation. 
#
labstoryInit <- function(labnoteDir="/home/mahogny/javaproj/labnote3.git/labnote"){
  .jinit()
    
  #actually, better to point to labnote.jar, and have it take the rest?
  
  #http://www.rforge.net/doc/packages/rJava/loader.html
  jardir<-file.path(labnoteDir,"libs")
  for(jarfile in list.files(jardir)){
    if(length(grep(".jar",jarfile))>0){
      #    print(file.path(jardir,jarfile))
      .jaddClassPath(file.path(jardir,jarfile))
    }
  }
  
  #This is for Labstory developers
  if(file.exists(file.path(labnoteDir,"bin"))){
    .jaddClassPath(file.path(labnoteDir,"bin"))
  }
  
  rLabstory<-.jnew(labstoryClass$RLabstory)
  return(rLabstory)
}


############################################
# Shut down Labstory
labstoryShutdown<-function(rLabstory){
  rLabstory$client$shutdown()
}

#.jclassPath()


#Thing to remember: SQL database will not allow two concurrent labstory instances


#TODO should not have a LIMS manager not connected to client! never static!






###############################################################
# Query for documents
labstoryQueryDocuments<-function(rLabstory, q=""){
  labstoryUtil <- .jnew(labstoryClass$RLabstoryUtil,rLabstory)
  .jcall(labstoryUtil, "[S", "getDocumentList")
}


###############################################################
# Construct a class as a return parameter
.jasret<-function(s){
  return(paste("L",s,";",sep=""))
}

############################################
# Load a document
labstoryLoadDocument<-function(documentID){
  labstoryUtil <- .jnew(labstoryClass$RLabstoryUtil,rLabstory)
  return(.jcall(labstoryUtil,
                .jasret(labstoryClass$Document),"loadDocument",documentID))
}



############################################
# Get a document object, by index
labstoryGetDocumentObjectByIndex<-function(doc, index){
  return(.jcall(doc,
                .jasret(labstoryClass$DocumentObject),"getObjectByIndex",as.integer(index)))
}

############################################
# Get number of document objects
labstoryGetDocumentObjectCount<-function(doc){
  return(.jcall(doc,
                "I","getObjectCount"))
}




