#################################################################
# Distribution functions

dt <- function(x, df, ncp, log = FALSE)                       .Primitive("dt")
pt <- function(q, df, ncp, lower.tail = TRUE, log.p = FALSE)  .Primitive("pt")
qt <- function(p, df, ncp, lower.tail = TRUE, log.p = FALSE)  .Primitive("qt")
rt <- function(n, df, ncp)                                    .Primitive("rt")

df <- function(x, df1, df2, ncp, log = FALSE)                       .Primitive("df")
pf <- function(q, df1, df2, ncp, lower.tail = TRUE, log.p = FALSE)  .Primitive("pf")
qf <- function(p, df1, df2, ncp, lower.tail = TRUE, log.p = FALSE)  .Primitive("qf")
rf <- function(n, df1, df2, ncp)                                    .Primitive("rf")

dchisq <- function(x, df, ncp = 0, log = FALSE)                            .Primitive("dchisq")
pchisq <- function(q, df, ncp = 0, lower.tail = TRUE, log.p = FALSE)       .Primitive("pchisq")
qchisq <- function(p, df, ncp = 0, lower.tail = TRUE, log.p = FALSE)       .Primitive("qchisq")
rchisq <- function(n, df, ncp = 0)                                         .Primitive("rchisq")

dnorm <- function(x, mean = 0, sd = 1, log = FALSE)                        .Primitive("dnorm")
pnorm <- function(q, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE)   .Primitive("pnorm")
qnorm <- function(p, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE)   .Primitive("qnorm")
rnorm <- function(n, mean=0, sd=1)                                         .Primitive("rnorm")


############ TODO below "###############

dbeta <- function(x, shape1, shape2, ncp = 0, log = FALSE)                        .Primitive("dbeta")
pbeta <- function(q, shape1, shape2, ncp = 0, lower.tail = TRUE, log.p = FALSE)   .Primitive("pbeta")
qbeta <- function(p, shape1, shape2, ncp = 0, lower.tail = TRUE, log.p = FALSE)   .Primitive("qbeta")
rbeta <- function(n, shape1, shape2, ncp = 0)                                     .Primitive("rbeta")

dbinom <- function(x, size, prob, log = FALSE)                        .Primitive("dbinom")
pbinom <- function(q, size, prob, lower.tail = TRUE, log.p = FALSE)   .Primitive("pbinom")
qbinom <- function(p, size, prob, lower.tail = TRUE, log.p = FALSE)   .Primitive("qbinom")
rbinom <- function(n, size, prob)                                     .Primitive("rbinom")

dexp <- function(x, rate = 1, log = FALSE)                       .Primitive("dexp")
pexp <- function(q, rate = 1, lower.tail = TRUE, log.p = FALSE)  .Primitive("pexp")
qexp <- function(p, rate = 1, lower.tail = TRUE, log.p = FALSE)  .Primitive("qexp")
rexp <- function(n, rate = 1)                                    .Primitive("rexp")

dgamma <- function(x, shape, rate = 1, scale = 1/rate, log = FALSE)                       .Primitive("dgamma")
pgamma <- function(q, shape, rate = 1, scale = 1/rate, lower.tail = TRUE, log.p = FALSE)  .Primitive("pgamma")
qgamma <- function(p, shape, rate = 1, scale = 1/rate, lower.tail = TRUE, log.p = FALSE)  .Primitive("qgamma")
rgamma <- function(n, shape, rate = 1, scale = 1/rate)                                    .Primitive("rgamma")

dgeom <- function(x, prob, log = FALSE)                      .Primitive("dgeom")
pgeom <- function(q, prob, lower.tail = TRUE, log.p = FALSE) .Primitive("pgeom")
qgeom <- function(p, prob, lower.tail = TRUE, log.p = FALSE) .Primitive("qgeom")
rgeom <- function(n, prob)                                   .Primitive("rgeom")

dhyper <- function(x, m, n, k, log = FALSE)                       .Primitive("dhyper")
phyper <- function(q, m, n, k, lower.tail = TRUE, log.p = FALSE)  .Primitive("phyper")
qhyper <- function(p, m, n, k, lower.tail = TRUE, log.p = FALSE)  .Primitive("qhyper")
rhyper <- function(nn, m, n, k)                                   .Primitive("rhyper")

dlogis <- function(x, location = 0, scale = 1, log = FALSE)                       .Primitive("dlogis")
plogis <- function(q, location = 0, scale = 1, lower.tail = TRUE, log.p = FALSE)  .Primitive("plogis")
qlogis <- function(p, location = 0, scale = 1, lower.tail = TRUE, log.p = FALSE)  .Primitive("qlogis")
rlogis <- function(n, location = 0, scale = 1)                                    .Primitive("rlogis")

dnbinom <- function(x, size, prob, mu, log = FALSE)                       .Primitive("dnbinom")
pnbinom <- function(q, size, prob, mu, lower.tail = TRUE, log.p = FALSE)  .Primitive("pnbinom")
qnbinom <- function(p, size, prob, mu, lower.tail = TRUE, log.p = FALSE)  .Primitive("qnbinom")
rnbinom <- function(n, size, prob, mu)                                    .Primitive("rnbinom")

dpois <- function(x, lambda, log = FALSE)                       .Primitive("dpois")
ppois <- function(q, lambda, lower.tail = TRUE, log.p = FALSE)  .Primitive("ppois")
qpois <- function(p, lambda, lower.tail = TRUE, log.p = FALSE)  .Primitive("qpois")
rpois <- function(n, lambda)                                    .Primitive("rpois")

############ TODO above "###############



#################################################################
# t-test
t.test <- function(x, y = NULL,
                   alternative = c("two.sided", "less", "greater"),
                   mu = 0, paired = FALSE, var.equal = FALSE,
                   conf.level = 0.95, ...) {
  
  t <- (mean(x) - mu)/(sd(x)/sqrt(length(x)))
  
  if(paired)
    stop("paired not implemented")
  
  if(!is.null(y))
    stop("y not implemented")
  # http://en.wikipedia.org/wiki/Welch%E2%80%93Satterthwaite_equation

  if(var.equal)
    stop("not implemented")
  #if var.equal is false...
  #the Welch (or Satterthwaite) approximation to the degrees of freedom is used.
  #otherwise pool variance
  
#  if(t<0)
#    t<- -t   ##eek! here a reference is not good enough!
  
  if(alternative[1]=="two.sided"){
  	tt <- abs(t)
#  	print("tt",t,tt)
    p.value <- pnorm(-tt) + pnorm(tt, lower.tail = FALSE)
  } else if(alternative[1]=="less"){
    p.value <- pnorm(t)
  } else if(alternative[1]=="greater"){
    p.value <- pnorm(t, lower.tail = FALSE)
  } else {
    stop("Undefined alternative")
  }
  
  # TODO assign a class, and override summary()
  p.value
}       
       
       
       
       
       
#################################################################
# 
wilcox.test <- function(x, y = NULL,
            alternative = c("two.sided", "less", "greater"),
            mu = 0, paired = FALSE, exact = NULL, correct = TRUE,
            conf.int = FALSE, conf.level = 0.95, ...)       
    .Primitive("wilcox.test")
    
    
    

#workspace = 2e+05    #TODO not parsing this right now!

function (x, y = NULL, workspace = 200000, hybrid = FALSE, control = list(), 
          or = 1, alternative = "two.sided", conf.int = TRUE, conf.level = 0.95, 
          simulate.p.value = FALSE, B = 2000) 
    stop("To implement")
    
#plenty of R code. requires some other routines        